@@ -239,7 +239,7 @@ g_io_channel_write (GIOChannel  *channel,
  **/
 GIOError 
 g_io_channel_seek  (GIOChannel   *channel,
-		    glong         offset, 
+		    gint64        offset, 
 		    GSeekType     type)
 {
   GError *err = NULL;
@@ -346,27 +346,35 @@ g_io_channel_shutdown (GIOChannel *channel,
   g_return_val_if_fail (channel != NULL, G_IO_STATUS_ERROR);
   g_return_val_if_fail (err == NULL || *err == NULL, G_IO_STATUS_ERROR);
 
-  if (flush && channel->write_buf && channel->write_buf->len > 0)
+  if (channel->write_buf && channel->write_buf->len > 0)
     {
-      GIOFlags flags;
-      
-      /* Set the channel to blocking, to avoid a busy loop
-       */
-      flags = g_io_channel_get_flags (channel);
-      /* Ignore any errors here, they're irrelevant */
-      g_io_channel_set_flags (channel, flags & ~G_IO_FLAG_NONBLOCK, NULL);
-
-      result = g_io_channel_flush (channel, &tmperr);
-
-      if (channel->partial_write_buf[0] != '\0')
+      if (flush)
         {
-          g_warning ("Partial character at end of write buffer not flushed.\n");
-          channel->partial_write_buf[0] = '\0';
+          GIOFlags flags;
+      
+          /* Set the channel to blocking, to avoid a busy loop
+           */
+          flags = g_io_channel_get_flags (channel);
+          /* Ignore any errors here, they're irrelevant */
+          g_io_channel_set_flags (channel, flags & ~G_IO_FLAG_NONBLOCK, NULL);
+
+          result = g_io_channel_flush (channel, &tmperr);
         }
+      else
+        result = G_IO_STATUS_NORMAL;
+
+      g_string_truncate(channel->write_buf, 0);
     }
   else
     result = G_IO_STATUS_NORMAL;
 
+  if (channel->partial_write_buf[0] != '\0')
+    {
+      if (flush)
+        g_warning ("Partial character at end of write buffer not flushed.\n");
+      channel->partial_write_buf[0] = '\0';
+    }
+
   status = channel->funcs->io_close (channel, err);
 
   channel->close_on_unref = FALSE; /* Because we already did */
@@ -727,7 +735,7 @@ g_io_channel_get_flags (GIOChannel *channel)
 {
   GIOFlags flags;
 
-  g_return_val_if_fail (channel != NULL, G_IO_STATUS_ERROR);
+  g_return_val_if_fail (channel != NULL, 0);
 
   flags = (* channel->funcs->io_get_flags) (channel);
 
@@ -799,7 +807,7 @@ g_io_channel_get_close_on_unref	(GIOChannel *channel)
  **/
 GIOStatus
 g_io_channel_seek_position	(GIOChannel* channel,
-				 glong       offset,
+				 gint64      offset,
 				 GSeekType   type,
 				 GError    **error)
 {
@@ -999,7 +1007,8 @@ g_io_channel_get_buffered	(GIOChannel *channel)
  *
  * The encoding %NULL is safe to use with binary data.
  *
- * The encoding can only be set under the following conditions:
+ * The encoding can only be set if one of the following conditions
+ * is true:
  *
  * 1. The channel was just created, and has not been written to
  *    or read from yet.
@@ -1022,9 +1031,9 @@ g_io_channel_get_buffered	(GIOChannel *channel)
  *    g_io_channel_read_line_string (), or g_io_channel_read_to_end ()
  *    does <emphasis>not</emphasis> guarantee that the encoding can be changed.
  *
- * Channels which do not meet the above conditions cannot call
+ * Channels which do not meet one of the above conditions cannot call
  * g_io_channel_seek_position () with an offset of %G_SEEK_CUR,
- * and if they are "seekable" cannot
+ * and, if they are "seekable", cannot
  * call g_io_channel_write_chars () after calling one
  * of the API "read" functions.
  *
@@ -1112,7 +1121,7 @@ g_io_channel_set_encoding (GIOChannel	*channel,
           else
             g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
                          _("Could not open converter from `%s' to `%s': %s"),
-                         from_enc, to_enc, strerror (err));
+                         from_enc, to_enc, g_strerror (err));
 
           if (read_cd != (GIConv) -1)
             g_iconv_close (read_cd);
@@ -1286,7 +1295,7 @@ reencode:
               default:
                 g_assert (errval != EBADF); /* The converter should be open */
                 g_set_error (err, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                  _("Error during conversion: %s"), strerror (errval));
+                  _("Error during conversion: %s"), g_strerror (errval));
                 return G_IO_STATUS_ERROR;
             }
         }
@@ -1348,7 +1357,7 @@ reencode:
 /**
  * g_io_channel_read_line:
  * @channel: a #GIOChannel
- * @str_return: The line read from the #GIOChannel, not including the
+ * @str_return: The line read from the #GIOChannel, including the
  *              line terminator. This data should be freed with g_free()
  *              when no longer needed. This is a nul-terminated string. 
  *              If a @length of zero is returned, this will be %NULL instead.
@@ -1359,11 +1368,10 @@ reencode:
  *
  * Reads a line, including the terminating character(s),
  * from a #GIOChannel into a newly-allocated string.
- * @length will contain allocated memory if the return
+ * @str_return will contain allocated memory if the return
  * is %G_IO_STATUS_NORMAL.
  *
- * Return value: a newly-allocated string. Free this string
- *   with g_free() when you are done with it.
+ * Return value: the status of the operation.
  **/
 GIOStatus
 g_io_channel_read_line (GIOChannel *channel,
@@ -1730,7 +1738,7 @@ g_io_channel_read_chars (GIOChannel	*channel,
 
   if (!channel->use_buffer)
     {
-      gint tmp_bytes;
+      gsize tmp_bytes;
       
       g_assert (!channel->read_buf || channel->read_buf->len == 0);
 
@@ -1922,7 +1930,7 @@ g_io_channel_write_chars (GIOChannel	*channel,
 
   if (!channel->use_buffer)
     {
-      gint tmp_bytes;
+      gsize tmp_bytes;
       
       g_assert (!channel->write_buf || channel->write_buf->len == 0);
       g_assert (channel->partial_write_buf[0] == '\0');
@@ -2172,7 +2180,7 @@ reconvert:
                     return G_IO_STATUS_ERROR;
                   default:
                     g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                      _("Error during conversion: %s"), strerror (errnum));
+                      _("Error during conversion: %s"), g_strerror (errnum));
                     if (from_buf_len >= left_len + from_buf_old_len)
                       wrote_bytes += from_buf_len - left_len - from_buf_old_len;
                     if (bytes_written)
