@@ -21,49 +21,36 @@
  * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GLib at ftp://ftp.gtk.org/pub/gtk/.
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-/*
+/* 
  * MT safe
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
+#include "glib.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <signal.h>
 #include <locale.h>
 #include <errno.h>
+#include "gdebug.h"
 
 #ifdef G_OS_WIN32
-#include <process.h>		/* For getpid() */
-#include <io.h>
-#  define STRICT		/* Strict typing, please */
-#  define _WIN32_WINDOWS 0x0401 /* to get IsDebuggerPresent */
-#  include <windows.h>
-#  undef STRICT
+typedef FILE* GFileDescriptor;
+#else
+typedef gint GFileDescriptor;
 #endif
 
-#include "gmessages.h"
-
-#include "gbacktrace.h"
-#include "gconvert.h"
-#include "gdebug.h"
-#include "gmem.h"
-#include "gprintfint.h"
-#include "gtestutils.h"
-#include "gthread.h"
-#include "gthreadprivate.h"
-#include "gstrfuncs.h"
-#include "gstring.h"
-
-
 /* --- structures --- */
 typedef struct _GLogDomain	GLogDomain;
 typedef struct _GLogHandler	GLogHandler;
@@ -84,6 +71,14 @@ struct _GLogHandler
 };
 
 
+/* --- prototypes --- */
+#ifndef HAVE_C99_VSNPRINTF
+static gsize printf_string_upper_bound (const gchar *format,
+					gboolean     may_warn,
+					va_list      args);
+#endif /* !HAVE_C99_VSNPRINTF */
+
+
 /* --- variables --- */
 static GMutex        *g_messages_lock = NULL;
 static GLogDomain    *g_log_domains = NULL;
@@ -92,28 +87,28 @@ static GPrintFunc     glib_print_func = NULL;
 static GPrintFunc     glib_printerr_func = NULL;
 static GPrivate	     *g_log_depth = NULL;
 static GLogLevelFlags g_log_msg_prefix = G_LOG_LEVEL_ERROR | G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_DEBUG;
-static GLogFunc       default_log_func = g_log_default_handler;
-static gpointer       default_log_data = NULL;
-static GTestLogFatalFunc fatal_log_func = NULL;
-static gpointer          fatal_log_data;
+
 
 /* --- functions --- */
 #ifdef G_OS_WIN32
 #  define STRICT
 #  include <windows.h>
 #  undef STRICT
+#  include <process.h>          /* For _getpid() */
+static gboolean alloc_console_called = FALSE;
 static gboolean win32_keep_fatal_message = FALSE;
 
 /* This default message will usually be overwritten. */
-/* Yes, a fixed size buffer is bad. So sue me. But g_error() is never
- * called with huge strings, is it?
+/* Yes, a fixed size buffer is bad. So sue me. But g_error is never
+ * with huge strings, is it?
  */
 static gchar  fatal_msg_buf[1000] = "Unspecified fatal error encountered, aborting.";
 static gchar *fatal_msg_ptr = fatal_msg_buf;
 
+/* Just use stdio. If we're out of memory, we're hosed anyway. */
 #undef write
 static inline int
-dowrite (int          fd,
+dowrite (GFileDescriptor fd,
 	 const void  *buf,
 	 unsigned int len)
 {
@@ -125,17 +120,40 @@ dowrite (int          fd,
       return len;
     }
 
-  write (fd, buf, len);
+  fwrite (buf, len, 1, fd);
+  fflush (fd);
 
   return len;
 }
 #define write(fd, buf, len) dowrite(fd, buf, len)
 
+static void
+ensure_stdout_valid (void)
+{
+  HANDLE handle;
+
+  if (win32_keep_fatal_message)
+    return;
+
+  if (!alloc_console_called)
+    {
+      handle = GetStdHandle (STD_OUTPUT_HANDLE);
+  
+      if (handle == INVALID_HANDLE_VALUE)
+	{
+	  AllocConsole ();
+	  alloc_console_called = TRUE;
+	  freopen ("CONOUT$", "w", stdout);
+	}
+    }
+}
+#else
+#define ensure_stdout_valid()	/* Define as empty */
 #endif
 
 static void
-write_string (int          fd,
-	      const gchar *string)
+write_string (GFileDescriptor fd,
+	      const gchar    *string)
 {
   write (fd, string, strlen (string));
 }
@@ -154,7 +172,7 @@ g_messages_prefixed_init (void)
       
       if (val)
 	{
-	  const GDebugKey keys[] = {
+	  static const GDebugKey keys[] = {
 	    { "error", G_LOG_LEVEL_ERROR },
 	    { "critical", G_LOG_LEVEL_CRITICAL },
 	    { "warning", G_LOG_LEVEL_WARNING },
@@ -248,9 +266,7 @@ g_log_domain_get_handler_L (GLogDomain	*domain,
 	  handler = handler->next;
 	}
     }
-
-  *data = default_log_data;
-  return default_log_func;
+  return g_log_default_handler;
 }
 
 GLogLevelFlags
@@ -341,54 +357,6 @@ g_log_set_handler (const gchar	 *log_domain,
   return handler_id;
 }
 
-GLogFunc
-g_log_set_default_handler (GLogFunc log_func,
-			   gpointer user_data)
-{
-  GLogFunc old_log_func;
-  
-  g_mutex_lock (g_messages_lock);
-  old_log_func = default_log_func;
-  default_log_func = log_func;
-  default_log_data = user_data;
-  g_mutex_unlock (g_messages_lock);
-  
-  return old_log_func;
-}
-
-/**
- * g_test_log_set_fatal_handler:
- * @log_func: the log handler function.
- * @user_data: data passed to the log handler.
- *
- * Installs a non-error fatal log handler which can be
- * used to decide whether log messages which are counted
- * as fatal abort the program.
- *
- * The use case here is that you are running a test case
- * that depends on particular libraries or circumstances
- * and cannot prevent certain known critical or warning
- * messages. So you install a handler that compares the
- * domain and message to precisely not abort in such a case.
- *
- * Note that the handler is reset at the beginning of
- * any test case, so you have to set it inside each test
- * function which needs the special behavior.
- *
- * This handler has no effect on g_error messages.
- *
- * Since: 2.22
- **/
-void
-g_test_log_set_fatal_handler (GTestLogFatalFunc log_func,
-                              gpointer          user_data)
-{
-  g_mutex_lock (g_messages_lock);
-  fatal_log_func = log_func;
-  fatal_log_data = user_data;
-  g_mutex_unlock (g_messages_lock);
-}
-
 void
 g_log_remove_handler (const gchar *log_domain,
 		      guint	   handler_id)
@@ -436,27 +404,50 @@ g_logv (const gchar   *log_domain,
 	const gchar   *format,
 	va_list	       args1)
 {
+  gchar buffer[1025];
   gboolean was_fatal = (log_level & G_LOG_FLAG_FATAL) != 0;
   gboolean was_recursion = (log_level & G_LOG_FLAG_RECURSION) != 0;
   gint i;
 
+#ifndef  HAVE_VSNPRINTF
+  va_list args2;
+#endif	/* HAVE_VSNPRINTF */
+  
   log_level &= G_LOG_LEVEL_MASK;
   if (!log_level)
     return;
-
+  
+  /* we use a stack buffer of fixed size, because we might get called
+   * recursively.
+   */
+#ifdef  HAVE_VSNPRINTF
+  vsnprintf (buffer, 1024, format, args1);
+#else	/* !HAVE_VSNPRINTF */
+  G_VA_COPY (args2, args1);
+  if (printf_string_upper_bound (format, FALSE, args1) < 1024)
+    vsprintf (buffer, format, args2);
+  else
+    {
+      /* since we might be out of memory, we can't use g_vsnprintf(). */
+      /* we are out of luck here */
+      strncpy (buffer, format, 1024);
+      buffer[1024] = 0;
+    }
+  va_end (args2);
+#endif	/* !HAVE_VSNPRINTF */
+  
   for (i = g_bit_nth_msf (log_level, -1); i >= 0; i = g_bit_nth_msf (log_level, i))
     {
       register GLogLevelFlags test_level;
-
+      
       test_level = 1 << i;
       if (log_level & test_level)
 	{
 	  guint depth = GPOINTER_TO_UINT (g_private_get (g_log_depth));
 	  GLogDomain *domain;
 	  GLogFunc log_func;
-	  GLogLevelFlags domain_fatal_mask;
+	  guint domain_fatal_mask;
 	  gpointer data = NULL;
-          gboolean masquerade_fatal = FALSE;
 
 	  if (was_fatal)
 	    test_level |= G_LOG_FLAG_FATAL;
@@ -484,7 +475,7 @@ g_logv (const gchar   *log_domain,
 	  /* had to defer debug initialization until we can keep track of recursion */
 	  if (!(test_level & G_LOG_FLAG_RECURSION) && !_g_debug_initialized)
 	    {
-	      GLogLevelFlags orig_test_level = test_level;
+	      guint orig_test_level = test_level;
 
 	      _g_debug_init ();
 	      if ((domain_fatal_mask | g_log_always_fatal) & test_level)
@@ -500,63 +491,21 @@ g_logv (const gchar   *log_domain,
 		}
 	    }
 
-	  if (test_level & G_LOG_FLAG_RECURSION)
+	  log_func (log_domain, test_level, buffer, data);
+
+	  if (test_level & G_LOG_FLAG_FATAL)
 	    {
-	      /* we use a stack buffer of fixed size, since we're likely
-	       * in an out-of-memory situation
-	       */
-	      gchar buffer[1025];
-              gsize size;
-              va_list args2;
-
-              G_VA_COPY (args2, args1);
-	      size = _g_vsnprintf (buffer, 1024, format, args2);
-              va_end (args2);
-
-	      log_func (log_domain, test_level, buffer, data);
-	    }
-	  else
-	    {
-	      gchar *msg;
-              va_list args2;
-
-              G_VA_COPY (args2, args1);
-              msg = g_strdup_vprintf (format, args2);
-              va_end (args2);
-
-	      log_func (log_domain, test_level, msg, data);
-
-              if ((test_level & G_LOG_FLAG_FATAL)
-                && !(test_level & G_LOG_LEVEL_ERROR))
-                {
-                  masquerade_fatal = fatal_log_func
-                    && !fatal_log_func (log_domain, test_level, msg, data);
-                }
-
-	      g_free (msg);
-	    }
-
-	  if ((test_level & G_LOG_FLAG_FATAL) && !masquerade_fatal)
-            {
 #ifdef G_OS_WIN32
-	      gchar *locale_msg = g_locale_from_utf8 (fatal_msg_buf, -1, NULL, NULL, NULL);
-	      
-	      MessageBox (NULL, locale_msg, NULL,
-			  MB_ICONERROR|MB_SETFOREGROUND);
-	      if (IsDebuggerPresent () && !(test_level & G_LOG_FLAG_RECURSION))
-		G_BREAKPOINT ();
-	      else
-		abort ();
-#else
-#if defined (G_ENABLE_DEBUG) && defined (SIGTRAP)
+	      MessageBox (NULL, fatal_msg_buf, NULL, MB_OK);
+#endif
+#if defined (G_ENABLE_DEBUG) && (defined (SIGTRAP) || defined (G_OS_WIN32))
 	      if (!(test_level & G_LOG_FLAG_RECURSION))
 		G_BREAKPOINT ();
 	      else
 		abort ();
-#else /* !G_ENABLE_DEBUG || !SIGTRAP */
+#else /* !G_ENABLE_DEBUG || !(SIGTRAP || G_OS_WIN32) */
 	      abort ();
-#endif /* !G_ENABLE_DEBUG || !SIGTRAP */
-#endif /* !G_OS_WIN32 */
+#endif /* !G_ENABLE_DEBUG || !(SIGTRAP || G_OS_WIN32) */
 	    }
 	  
 	  depth--;
@@ -578,83 +527,12 @@ g_log (const gchar   *log_domain,
   va_end (args);
 }
 
-void
-g_return_if_fail_warning (const char *log_domain,
-			  const char *pretty_function,
-			  const char *expression)
-{
-  g_log (log_domain,
-	 G_LOG_LEVEL_CRITICAL,
-	 "%s: assertion `%s' failed",
-	 pretty_function,
-	 expression);
-}
-
-void
-g_warn_message (const char     *domain,
-                const char     *file,
-                int             line,
-                const char     *func,
-                const char     *warnexpr)
-{
-  char *s, lstr[32];
-  g_snprintf (lstr, 32, "%d", line);
-  if (warnexpr)
-    s = g_strconcat ("(", file, ":", lstr, "):",
-                     func, func[0] ? ":" : "",
-                     " runtime check failed: (", warnexpr, ")", NULL);
-  else
-    s = g_strconcat ("(", file, ":", lstr, "):",
-                     func, func[0] ? ":" : "",
-                     " ", "code should not be reached", NULL);
-  g_log (domain, G_LOG_LEVEL_WARNING, "%s", s);
-  g_free (s);
-}
-
-void
-g_assert_warning (const char *log_domain,
-		  const char *file,
-		  const int   line,
-		  const char *pretty_function,
-		  const char *expression)
-{
-  g_log (log_domain,
-	 G_LOG_LEVEL_ERROR,
-	 expression 
-	 ? "file %s: line %d (%s): assertion failed: (%s)"
-	 : "file %s: line %d (%s): should not be reached",
-	 file, 
-	 line, 
-	 pretty_function,
-	 expression);
-  abort ();
-}
-
-#define CHAR_IS_SAFE(wc) (!((wc < 0x20 && wc != '\t' && wc != '\n' && wc != '\r') || \
-			    (wc == 0x7f) || \
-			    (wc >= 0x80 && wc < 0xa0)))
-     
 static gchar*
 strdup_convert (const gchar *string,
 		const gchar *charset)
 {
   if (!g_utf8_validate (string, -1, NULL))
-    {
-      GString *gstring = g_string_new ("[Invalid UTF-8] ");
-      guchar *p;
-
-      for (p = (guchar *)string; *p; p++)
-	{
-	  if (CHAR_IS_SAFE(*p) &&
-	      !(*p == '\r' && *(p + 1) != '\n') &&
-	      *p < 0x80)
-	    g_string_append_c (gstring, *p);
-	  else
-	    g_string_append_printf (gstring, "\\x%02x", (guint)(guchar)*p);
-	}
-      
-      return g_string_free (gstring, FALSE);
-    }
+    return g_strconcat ("[Invalid UTF-8] ", string, NULL);
   else
     {
       GError *err = NULL;
@@ -670,7 +548,7 @@ strdup_convert (const gchar *string,
 	  if (!warned)
 	    {
 	      warned = TRUE;
-	      _g_fprintf (stderr, "GLib: Cannot convert message: %s\n", err->message);
+	      fprintf (stderr, "GLib: Cannot convert message: %s\n", err->message);
 	    }
 	  g_error_free (err);
 	  
@@ -755,9 +633,9 @@ format_unsigned (gchar  *buf,
 
 #define	ALERT_LEVELS		(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING)
 
-static int
-mklevel_prefix (gchar          level_prefix[STRING_BUFFER_SIZE],
-		GLogLevelFlags log_level)
+static GFileDescriptor
+mklevel_prefix (gchar level_prefix[STRING_BUFFER_SIZE],
+		guint log_level)
 {
   gboolean to_stdout = TRUE;
 
@@ -802,10 +680,14 @@ mklevel_prefix (gchar          level_prefix[STRING_BUFFER_SIZE],
   if (log_level & ALERT_LEVELS)
     strcat (level_prefix, " **");
 
+  ensure_stdout_valid ();
 #ifdef G_OS_WIN32
   win32_keep_fatal_message = (log_level & G_LOG_FLAG_FATAL) != 0;
-#endif
+  /* Use just stdout as stderr is hard to get redirected from the DOS prompt. */
+  return stdout;
+#else
   return to_stdout ? 1 : 2;
+#endif
 }
 
 void
@@ -814,12 +696,9 @@ _g_log_fallback_handler (const gchar   *log_domain,
 			 const gchar   *message,
 			 gpointer       unused_data)
 {
-  gchar level_prefix[STRING_BUFFER_SIZE];
-#ifndef G_OS_WIN32
-  gchar pid_string[FORMAT_UNSIGNED_BUFSIZE];
-#endif
+  gchar level_prefix[STRING_BUFFER_SIZE], pid_string[FORMAT_UNSIGNED_BUFSIZE];
   gboolean is_fatal = (log_level & G_LOG_FLAG_FATAL) != 0;
-  int fd;
+  GFileDescriptor fd;
 
   /* we can not call _any_ GLib functions in this fallback handler,
    * which is why we skip UTF-8 conversion, etc.
@@ -832,21 +711,15 @@ _g_log_fallback_handler (const gchar   *log_domain,
   if (!message)
     message = "(NULL) message";
 
-#ifndef G_OS_WIN32
   format_unsigned (pid_string, getpid (), 10);
-#endif
 
   if (log_domain)
     write_string (fd, "\n");
   else
     write_string (fd, "\n** ");
-
-#ifndef G_OS_WIN32
   write_string (fd, "(process:");
   write_string (fd, pid_string);
   write_string (fd, "): ");
-#endif
-
   if (log_domain)
     {
       write_string (fd, log_domain);
@@ -861,66 +734,6 @@ _g_log_fallback_handler (const gchar   *log_domain,
     write_string (fd, "\n");
 }
 
-static void
-escape_string (GString *string)
-{
-  const char *p = string->str;
-  gunichar wc;
-
-  while (p < string->str + string->len)
-    {
-      gboolean safe;
-	    
-      wc = g_utf8_get_char_validated (p, -1);
-      if (wc == (gunichar)-1 || wc == (gunichar)-2)  
-	{
-	  gchar *tmp;
-	  guint pos;
-
-	  pos = p - string->str;
-
-	  /* Emit invalid UTF-8 as hex escapes 
-           */
-	  tmp = g_strdup_printf ("\\x%02x", (guint)(guchar)*p);
-	  g_string_erase (string, pos, 1);
-	  g_string_insert (string, pos, tmp);
-
-	  p = string->str + (pos + 4); /* Skip over escape sequence */
-
-	  g_free (tmp);
-	  continue;
-	}
-      if (wc == '\r')
-	{
-	  safe = *(p + 1) == '\n';
-	}
-      else
-	{
-	  safe = CHAR_IS_SAFE (wc);
-	}
-      
-      if (!safe)
-	{
-	  gchar *tmp;
-	  guint pos;
-
-	  pos = p - string->str;
-	  
-	  /* Largest char we escape is 0x0a, so we don't have to worry
-	   * about 8-digit \Uxxxxyyyy
-	   */
-	  tmp = g_strdup_printf ("\\u%04x", wc); 
-	  g_string_erase (string, pos, g_utf8_next_char (p) - p);
-	  g_string_insert (string, pos, tmp);
-	  g_free (tmp);
-
-	  p = string->str + (pos + 6); /* Skip over escape sequence */
-	}
-      else
-	p = g_utf8_next_char (p);
-    }
-}
-
 void
 g_log_default_handler (const gchar   *log_domain,
 		       GLogLevelFlags log_level,
@@ -930,7 +743,7 @@ g_log_default_handler (const gchar   *log_domain,
   gboolean is_fatal = (log_level & G_LOG_FLAG_FATAL) != 0;
   gchar level_prefix[STRING_BUFFER_SIZE], *string;
   GString *gstring;
-  int fd;
+  GFileDescriptor fd;
 
   /* we can be called externally with recursion for whatever reason */
   if (log_level & G_LOG_FLAG_RECURSION)
@@ -943,7 +756,7 @@ g_log_default_handler (const gchar   *log_domain,
 
   fd = mklevel_prefix (level_prefix, log_level);
 
-  gstring = g_string_new (NULL);
+  gstring = g_string_new ("");
   if (log_level & ALERT_LEVELS)
     g_string_append (gstring, "\n");
   if (!log_domain)
@@ -971,22 +784,16 @@ g_log_default_handler (const gchar   *log_domain,
     g_string_append (gstring, "(NULL) message");
   else
     {
-      GString *msg;
       const gchar *charset;
 
-      msg = g_string_new (message);
-      escape_string (msg);
-
       if (g_get_charset (&charset))
-	g_string_append (gstring, msg->str);	/* charset is UTF-8 already */
+	g_string_append (gstring, message);	/* charset is UTF-8 already */
       else
 	{
-	  string = strdup_convert (msg->str, charset);
+	  string = strdup_convert (message, charset);
 	  g_string_append (gstring, string);
 	  g_free (string);
 	}
-
-      g_string_free (msg, TRUE);
     }
   if (is_fatal)
     g_string_append (gstring, "\naborting...\n");
@@ -1036,6 +843,7 @@ g_print (const gchar *format,
     {
       const gchar *charset;
 
+      ensure_stdout_valid ();
       if (g_get_charset (&charset))
 	fputs (string, stdout); /* charset is UTF-8 already */
       else
@@ -1101,16 +909,337 @@ g_printerr (const gchar *format,
   g_free (string);
 }
 
+#ifndef MB_LEN_MAX
+#  define MB_LEN_MAX 8
+#endif
+
+#ifndef HAVE_C99_VSNPRINTF
+
+typedef struct
+{
+  guint min_width;
+  guint precision;
+  gboolean alternate_format, zero_padding, adjust_left, locale_grouping;
+  gboolean add_space, add_sign, possible_sign, seen_precision;
+  gboolean mod_half, mod_long, mod_extra_long;
+} PrintfArgSpec;
+
+static gsize
+printf_string_upper_bound (const gchar *format,
+			   gboolean     may_warn,
+			   va_list      args)
+{
+  static const gboolean honour_longs = SIZEOF_LONG > 4 || SIZEOF_VOID_P > 4;
+  gsize len = 1;
+  
+  if (!format)
+    return len;
+  
+  while (*format)
+    {
+      register gchar c = *format++;
+      
+      if (c != '%')
+	len += 1;
+      else /* (c == '%') */
+	{
+	  PrintfArgSpec spec = { 0, };
+	  gboolean seen_l = FALSE, conv_done = FALSE;
+	  gsize conv_len = 0;
+	  const gchar *spec_start = format;
+	  
+	  do
+	    {
+	      c = *format++;
+	      switch (c)
+		{
+		  GDoubleIEEE754 u_double;
+		  guint v_uint;
+		  gint v_int;
+		  const gchar *v_string;
+		  
+		  /* beware of positional parameters
+		   */
+		case '$':
+		  if (may_warn)
+		    g_warning (G_GNUC_PRETTY_FUNCTION
+			       "(): unable to handle positional parameters (%%n$)");
+		  len += 1024; /* try adding some safety padding */
+		  conv_done = TRUE;
+		  break;
+		  
+		  /* parse flags
+		   */
+		case '#':
+		  spec.alternate_format = TRUE;
+		  break;
+		case '0':
+		  spec.zero_padding = TRUE;
+		  break;
+		case '-':
+		  spec.adjust_left = TRUE;
+		  break;
+		case ' ':
+		  spec.add_space = TRUE;
+		  break;
+		case '+':
+		  spec.add_sign = TRUE;
+		  break;
+		case '\'':
+		  spec.locale_grouping = TRUE;
+		  break;
+		  
+		  /* parse output size specifications
+		   */
+		case '.':
+		  spec.seen_precision = TRUE;
+		  break;
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		case '8':
+		case '9':
+		  v_uint = c - '0';
+		  c = *format;
+		  while (c >= '0' && c <= '9')
+		    {
+		      format++;
+		      v_uint = v_uint * 10 + c - '0';
+		      c = *format;
+		    }
+		  if (spec.seen_precision)
+		    spec.precision = MAX (spec.precision, v_uint);
+		  else
+		    spec.min_width = MAX (spec.min_width, v_uint);
+		  break;
+		case '*':
+		  v_int = va_arg (args, int);
+		  if (spec.seen_precision)
+		    {
+		      /* forget about negative precision */
+		      if (v_int >= 0)
+			spec.precision = MAX (spec.precision, v_int);
+		    }
+		  else
+		    {
+		      if (v_int < 0)
+			{
+			  v_int = - v_int;
+			  spec.adjust_left = TRUE;
+			}
+		      spec.min_width = MAX (spec.min_width, v_int);
+		    }
+		  break;
+		  
+		  /* parse type modifiers
+		   */
+		case 'h':
+		  spec.mod_half = TRUE;
+		  break;
+		case 'l':
+		  if (!seen_l)
+		    {
+		      spec.mod_long = TRUE;
+		      seen_l = TRUE;
+		      break;
+		    }
+		  /* else, fall through */
+		case 'L':
+		case 'q':
+		  spec.mod_long = TRUE;
+		  spec.mod_extra_long = TRUE;
+		  break;
+		case 'z':
+		case 'Z':
+#if GLIB_SIZEOF_SIZE_T > 4
+		  spec.mod_long = TRUE;
+		  spec.mod_extra_long = TRUE;
+#endif /* GLIB_SIZEOF_SIZE_T > 4 */
+		  break;
+		case 't':
+#if GLIB_SIZEOF_PTRDIFF_T > 4
+		  spec.mod_long = TRUE;
+		  spec.mod_extra_long = TRUE;
+#endif /* GLIB_SIZEOF_PTRDIFF_T > 4 */
+		  break;
+		case 'j':
+#if GLIB_SIZEOF_INTMAX_T > 4
+		  spec.mod_long = TRUE;
+		  spec.mod_extra_long = TRUE;
+#endif /* GLIB_SIZEOF_INTMAX_T > 4 */
+		  break;
+		  
+		  /* parse output conversions
+		   */
+		case '%':
+		  conv_len += 1;
+		  break;
+		case 'O':
+		case 'D':
+		case 'I':
+		case 'U':
+		  /* some C libraries feature long variants for these as well? */
+		  spec.mod_long = TRUE;
+		  /* fall through */
+		case 'o':
+		  conv_len += 2;
+		  /* fall through */
+		case 'd':
+		case 'i':
+		  conv_len += 1; /* sign */
+		  /* fall through */
+		case 'u':
+		  conv_len += 4;
+		  /* fall through */
+		case 'x':
+		case 'X':
+		  spec.possible_sign = TRUE;
+		  conv_len += 10;
+		  if (spec.mod_long && honour_longs)
+		    conv_len *= 2;
+		  if (spec.mod_extra_long)
+		    conv_len *= 2;
+		  if (spec.mod_extra_long)
+		    {
+		      (void) va_arg (args, gint64);
+		    }
+		  else if (spec.mod_long)
+		    (void) va_arg (args, long);
+		  else
+		    (void) va_arg (args, int);
+		  break;
+		case 'A':
+		case 'a':
+		  /*          0x */
+		  conv_len += 2;
+		  /* fall through */
+		case 'g':
+		case 'G':
+		case 'e':
+		case 'E':
+		case 'f':
+		  spec.possible_sign = TRUE;
+		  /*          n   .   dddddddddddddddddddddddd   E   +-  eeee */
+		  conv_len += 1 + 1 + MAX (24, spec.precision) + 1 + 1 + 4;
+                  if (may_warn && spec.mod_extra_long)
+		    g_warning (G_GNUC_PRETTY_FUNCTION
+			       "(): unable to handle long double, collecting double only");
+#ifdef HAVE_LONG_DOUBLE
+#error need to implement special handling for long double
+#endif
+		  u_double.v_double = va_arg (args, double);
+		  /* %f can expand up to all significant digits before '.' (308) */
+		  if (c == 'f' &&
+		      u_double.mpn.biased_exponent > 0 && u_double.mpn.biased_exponent < 2047)
+		    {
+		      gint exp = u_double.mpn.biased_exponent;
+		      
+		      exp -= G_IEEE754_DOUBLE_BIAS;
+		      exp = exp * G_LOG_2_BASE_10 + 1;
+		      conv_len += ABS (exp);	/* exp can be <0 */
+		    }
+		  /* some printf() implementations require extra padding for rounding */
+		  conv_len += 2;
+		  /* we can't really handle locale specific grouping here */
+		  if (spec.locale_grouping)
+		    conv_len *= 2;
+		  break;
+		case 'C':
+		  spec.mod_long = TRUE;
+                  /* fall through */
+		case 'c':
+		  conv_len += spec.mod_long ? MB_LEN_MAX : 1;
+		  (void) va_arg (args, int);
+		  break;
+		case 'S':
+		  spec.mod_long = TRUE;
+		  /* fall through */
+		case 's':
+		  v_string = va_arg (args, char*);
+		  if (!v_string)
+		    conv_len += 8; /* hold "(null)" */
+		  else if (spec.seen_precision)
+		    conv_len += spec.precision;
+		  else
+		    conv_len += strlen (v_string);
+		  conv_done = TRUE;
+		  if (spec.mod_long)
+		    {
+		      if (may_warn)
+			g_warning (G_GNUC_PRETTY_FUNCTION
+				   "(): unable to handle wide char strings");
+		      len += 1024; /* try adding some safety padding */
+		    }
+		  break;
+		case 'P': /* do we actually need this? */
+		  /* fall through */
+		case 'p':
+		  spec.alternate_format = TRUE;
+		  conv_len += 10;
+		  if (honour_longs)
+		    conv_len *= 2;
+		  /* fall through */
+		case 'n':
+		  conv_done = TRUE;
+		  (void) va_arg (args, void*);
+		  break;
+		case 'm':
+		  /* there's not much we can do to be clever */
+		  v_string = g_strerror (errno);
+		  v_uint = v_string ? strlen (v_string) : 0;
+		  conv_len += MAX (256, v_uint);
+		  break;
+		  
+		  /* handle invalid cases
+		   */
+		case '\000':
+		  /* no conversion specification, bad bad */
+		  conv_len += format - spec_start;
+		  break;
+		default:
+		  if (may_warn)
+		    g_warning (G_GNUC_PRETTY_FUNCTION
+			       "(): unable to handle `%c' while parsing format",
+			       c);
+		  break;
+		}
+	      conv_done |= conv_len > 0;
+	    }
+	  while (!conv_done);
+	  /* handle width specifications */
+	  conv_len = MAX (conv_len, MAX (spec.precision, spec.min_width));
+	  /* handle flags */
+	  conv_len += spec.alternate_format ? 2 : 0;
+	  conv_len += (spec.add_space || spec.add_sign || spec.possible_sign);
+	  /* finally done */
+	  len += conv_len;
+	} /* else (c == '%') */
+    } /* while (*format) */
+  
+  return len;
+}
+
+#endif /* !HAVE_C99_VSNPRINTF */
+
+
 gsize
 g_printf_string_upper_bound (const gchar *format,
 			     va_list      args)
 {
+#if HAVE_C99_VSNPRINTF
   gchar c;
-  return _g_vsnprintf (&c, 1, format, args) + 1;
+  return vsnprintf (&c, 1, format, args) + 1;
+#else
+  return printf_string_upper_bound (format, TRUE, args);
+#endif
 }
 
 void
-_g_messages_thread_init_nomessage (void)
+g_messages_init (void)
 {
   g_messages_lock = g_mutex_new ();
   g_log_depth = g_private_new (NULL);
@@ -1131,9 +1260,8 @@ _g_debug_init (void)
   val = g_getenv ("G_DEBUG");
   if (val != NULL)
     {
-      const GDebugKey keys[] = {
-	{"fatal_warnings", G_DEBUG_FATAL_WARNINGS},
-	{"fatal_criticals", G_DEBUG_FATAL_CRITICALS}
+      static const GDebugKey keys[] = {
+	{"fatal_warnings", G_DEBUG_FATAL_WARNINGS}
       };
       
       _g_debug_flags = g_parse_debug_string (val, keys, G_N_ELEMENTS (keys));
@@ -1147,13 +1275,4 @@ _g_debug_init (void)
       fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
       g_log_set_always_fatal (fatal_mask);
     }
-  
-  if (_g_debug_flags & G_DEBUG_FATAL_CRITICALS) 
-    {
-      GLogLevelFlags fatal_mask;
-      
-      fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
-      fatal_mask |= G_LOG_LEVEL_CRITICAL;
-      g_log_set_always_fatal (fatal_mask);
-    }
 }
