@@ -32,13 +32,10 @@
  * at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#include "config.h"
-
 #include <string.h>
 
 #include "glib.h"
 
-
 /* Byte-wise swap two items of size SIZE. */
 #define SWAP(a, b, size)						      \
   do									      \
@@ -111,7 +108,7 @@ stack_node;
 void
 g_qsort_with_data (gconstpointer    pbase,
 		   gint             total_elems,
-		   gsize            size,
+		   size_t           size,
 		   GCompareDataFunc compare_func,
 		   gpointer         user_data)
 {
